<?php

namespace App\Service;

use App\Dto\DtoCollection;
use App\Dto\DtoInterface;
use App\Dto\KitchenTypeDto;
use App\Dto\PaginationDto;
use App\Dto\RestaurantFilterVariantsDto;
use App\Dto\RestaurantListDto;
use App\Dto\RestaurantListingElementDto;
use App\Dto\RestaurantTypeDto;
use App\Entity\Kitchens;
use App\Entity\Restaurants;
use App\Entity\RestaurantTypes;
use App\Repository\KitchensRepositoryInterface;
use App\Repository\RestaurantsRepositoryInterface;
use App\Repository\RestaurantTypeRepositoryInterface;
use App\Request\AbstractRequest;

class RestaurantListingService implements ServiceInterface
{
    public function __construct(
        private readonly RestaurantsRepositoryInterface $restaurants,
        private readonly RestaurantTypeRepositoryInterface $types,
        private readonly KitchensRepositoryInterface $kitchens,
    ) {}

    public function serve(AbstractRequest $request): DtoInterface
    {
        $countOfRestaurants = $this
            ->restaurants
            ->getCountWithFilters(
                $request->restaurant_type_id,
                $request->kitchen_id
            );

        $pagination = new PaginationDto(
            $request->page,
            $request->limit,
            $countOfRestaurants,
        );

        $offset = min($pagination->page_size, $countOfRestaurants) * ($pagination->current_page - 1);

        $list = new DtoCollection(
            RestaurantListingElementDto::class,
            $this
                ->restaurants
                ->findByFilters(
                    $request
                        ->restaurant_type_id,
                    $request->kitchen_id,
                    $request->limit,
                    $offset,
                )
                ->map(function (Restaurants $restaurant) {
                    return $restaurant->getDto();
                })
                ->toArray()
        );

        $filters = new RestaurantFilterVariantsDto(
            new DtoCollection(
                RestaurantTypeDto::class,
                $this->types->getAll()
                    ->map(function(RestaurantTypes $types) {
                        return $types->getDto();
                    })
                    ->toArray()
            ),
            new DtoCollection(
                KitchenTypeDto::class,
                $this->kitchens->getAll()
                    ->map(function(Kitchens $kitchen) {
                        return $kitchen->getDto();
                    })
                    ->toArray()
            ),
        );

        return new RestaurantListDto(
            $pagination,
            $list,
            $filters,
        );
    }
}
